#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['b2luigi',
 'b2luigi.basf2_helper',
 'b2luigi.batch',
 'b2luigi.batch.processes',
 'b2luigi.cli',
 'b2luigi.core']

package_data = \
{'': ['*'],
 'b2luigi.batch.processes': ['.mypy_cache/*',
                             '.mypy_cache/3.6/*',
                             '.mypy_cache/3.6/_typeshed/*',
                             '.mypy_cache/3.6/b2luigi/*',
                             '.mypy_cache/3.6/b2luigi/batch/*',
                             '.mypy_cache/3.6/b2luigi/cli/*',
                             '.mypy_cache/3.6/b2luigi/core/*',
                             '.mypy_cache/3.6/collections/*',
                             '.mypy_cache/3.6/importlib/*',
                             '.mypy_cache/3.6/json/*',
                             '.mypy_cache/3.6/os/*',
                             '.mypy_cache/3.8/*',
                             '.mypy_cache/3.8/_typeshed/*',
                             '.mypy_cache/3.8/b2luigi/*',
                             '.mypy_cache/3.8/b2luigi/batch/*',
                             '.mypy_cache/3.8/b2luigi/cli/*',
                             '.mypy_cache/3.8/b2luigi/core/*',
                             '.mypy_cache/3.8/collections/*',
                             '.mypy_cache/3.8/importlib/*',
                             '.mypy_cache/3.8/json/*',
                             '.mypy_cache/3.8/logging/*',
                             '.mypy_cache/3.8/os/*',
                             'gbasf2_utils/*',
                             'gbasf2_utils/.mypy_cache/*',
                             'gbasf2_utils/.mypy_cache/3.9/*',
                             'gbasf2_utils/.mypy_cache/3.9/_typeshed/*',
                             'gbasf2_utils/.mypy_cache/3.9/collections/*',
                             'gbasf2_utils/.mypy_cache/3.9/importlib/*',
                             'gbasf2_utils/.mypy_cache/3.9/os/*',
                             'templates/*'],
 'b2luigi.cli': ['.mypy_cache/*',
                 '.mypy_cache/3.9/*',
                 '.mypy_cache/3.9/_typeshed/*',
                 '.mypy_cache/3.9/asyncio/*',
                 '.mypy_cache/3.9/b2luigi/*',
                 '.mypy_cache/3.9/b2luigi/batch/*',
                 '.mypy_cache/3.9/b2luigi/cli/*',
                 '.mypy_cache/3.9/b2luigi/core/*',
                 '.mypy_cache/3.9/cachetools/*',
                 '.mypy_cache/3.9/collections/*',
                 '.mypy_cache/3.9/concurrent/*',
                 '.mypy_cache/3.9/concurrent/futures/*',
                 '.mypy_cache/3.9/ctypes/*',
                 '.mypy_cache/3.9/html/*',
                 '.mypy_cache/3.9/importlib/*',
                 '.mypy_cache/3.9/jinja2/*',
                 '.mypy_cache/3.9/json/*',
                 '.mypy_cache/3.9/logging/*',
                 '.mypy_cache/3.9/markupsafe/*',
                 '.mypy_cache/3.9/multiprocessing/*',
                 '.mypy_cache/3.9/os/*',
                 '.mypy_cache/3.9/urllib/*'],
 'b2luigi.core': ['.mypy_cache/*',
                  '.mypy_cache/3.8/*',
                  '.mypy_cache/3.8/_typeshed/*',
                  '.mypy_cache/3.8/b2luigi/*',
                  '.mypy_cache/3.8/b2luigi/batch/*',
                  '.mypy_cache/3.8/b2luigi/cli/*',
                  '.mypy_cache/3.8/b2luigi/core/*',
                  '.mypy_cache/3.8/collections/*',
                  '.mypy_cache/3.8/importlib/*',
                  '.mypy_cache/3.8/json/*',
                  '.mypy_cache/3.8/os/*']}

install_requires = \
['luigi>=3.0.2',
 'parse>=1.8.4',
 'GitPython>=2.1.11',
 'colorama>=0.3.9',
 'cachetools>=2.1.0',
 'jinja2']

setup(name='b2luigi',
      version='0.7.2',
      description='Task scheduling and batch running for basf2 jobs made simple',
      author='Nils Braun',
      author_email='nils.braun@kit.edu',
      url='https://github.com/nils-braun/b2luigi',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
